@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Based on work Copyright (C) 1998 - 2007 Tenable Network Security, Inc.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -23,8 +23,8 @@
  * @brief Header file for module bpf_share.
  */
 
-#ifndef OPENVAS_BPF_SHARE_H
-#define OPENVAS_BPF_SHARE_H
+#ifndef MISC_BPF_SHARE_H
+#define MISC_BPF_SHARE_H
 
 #include <sys/types.h>
 
