@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Based on work Copyright (C) 1998 Renaud Deraison
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -23,8 +23,8 @@
  * @brief Header file for module ftp_funcs.
  */
 
-#ifndef OPENVAS_FTP_FUNCS_H
-#define OPENVAS_FTP_FUNCS_H
+#ifndef MISC_FTP_FUNCS_H
+#define MISC_FTP_FUNCS_H
 
 #include <arpa/inet.h>
 #include <sys/param.h>
