@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Based on work Copyright (C) 1998 - 2002 Renaud Deraison
  *               SSL Support Copyright (C) 2001 Michel Arboi
  *
@@ -25,6 +25,7 @@
  */
 
 #include "../nasl/nasl_debug.h" /* for nasl_*_filename */
+#include "kb_cache.h"
 
 #include <arpa/inet.h> /* for inet_pton */
 #include <errno.h>
@@ -156,6 +157,9 @@ stream_get_err (int fd)
   return p->last_err;
 }
 
+const char *tls_priorities = "NORMAL:+ARCFOUR-128:%COMPAT";
+int tls_priority_flag = NO_PRIORITY_FLAGS;
+
 /**
  * @brief Returns a free file descriptor.
  */
@@ -332,7 +336,7 @@ block_socket (int soc)
  * default value: 1 according to SVID 3, BSD 4.3, ISO 9899 :-(
  */
 
-void
+static void
 tlserror (char *txt, int err)
 {
   g_message ("[%d] %s: %s", getpid (), txt, gnutls_strerror (err));
@@ -438,6 +442,8 @@ set_gnutls_protocol (gnutls_session_t session, openvas_encaps_t encaps,
       break;
     }
 
+  g_debug ("%s: setting %s as priority_string based on %d", __func__,
+           priorities, encaps);
   if ((err = gnutls_priority_set_direct (session, priorities, &errloc)))
     {
       g_message ("[%d] setting session priorities '%.20s': %s", getpid (),
@@ -863,9 +869,12 @@ socket_negotiate_ssl (int fd, openvas_encaps_t transport,
       if (!connection_failed_msg_sent)
         {
           g_message ("Function socket_negotiate_ssl called from %s: "
-                     "SSL/TLS connection (host: %s) failed.",
+                     "SSL/TLS connection (host: %s, ip: %s) failed.",
                      nasl_get_plugin_filename (),
-                     hostname ? hostname : "unknown");
+                     plug_get_host_fqdn (args) ? plug_get_host_fqdn (args)
+                                               : "unknown",
+                     plug_get_host_ip_str (args) ? plug_get_host_ip_str (args)
+                                                 : "unknown");
           connection_failed_msg_sent = TRUE;
         }
       g_free (hostname);
@@ -887,6 +896,7 @@ socket_negotiate_ssl (int fd, openvas_encaps_t transport,
  * @param[out]  cert    Memory pointer to fill cert pointer.
  * @param[out]  certlen Size of cert.
  */
+
 void
 socket_get_cert (int fd, void **cert, int *certlen)
 {
@@ -1168,13 +1178,19 @@ failed:
   return ret;
 }
 
+void
+open_stream_tls_default_priorities (const char *p, const int pflag)
+{
+  tls_priorities = p;
+  tls_priority_flag = pflag;
+}
+
 int
 open_stream_connection (struct script_infos *args, unsigned int port,
                         int transport, int timeout)
 {
   return open_stream_connection_ext (args, port, transport, timeout,
-                                     "NORMAL:+ARCFOUR-128:%COMPAT",
-                                     NO_PRIORITY_FLAGS);
+                                     tls_priorities, tls_priority_flag);
 }
 
 /* Same as open_stream_auto_encaps but allows to force auto detection
@@ -1378,7 +1394,8 @@ read_stream_connection_unbuffered (int fd, void *buf0, int min_len, int max_len)
       if (fp->transport || fp->fd != 0)
         g_message ("Function %s (calling internal function %s) called from %s: "
                    "Severe bug! Unhandled transport layer %d (fd=%d).",
-                   nasl_get_function_name () ?: "script_main_function",
+                   nasl_get_function_name () ? nasl_get_function_name ()
+                                             : "script_main_function",
                    __func__, nasl_get_plugin_filename (), fp->transport, fd);
       else
         g_message ("read_stream_connection_unbuffered: "
@@ -1559,7 +1576,8 @@ write_stream_connection4 (int fd, void *buf0, int n, int i_opt)
       if (fp->transport || fp->fd != 0)
         g_message ("Function %s (calling internal function %s) called from %s: "
                    "Severe bug! Unhandled transport layer %d (fd=%d).",
-                   nasl_get_function_name () ?: "script_main_function",
+                   nasl_get_function_name () ? nasl_get_function_name ()
+                                             : "script_main_function",
                    __func__, nasl_get_plugin_filename (), fp->transport, fd);
       else
         g_message ("read_stream_connection_unbuffered: fd=%d is "
@@ -1953,7 +1971,7 @@ open_sock_tcp (struct script_infos *args, unsigned int port, int timeout)
         {
           g_message ("open_sock_tcp: %s:%d time-out.", ip_str, port);
           log_count++;
-          kb_item_set_int (kb, buffer, log_count);
+          kb_item_set_int_with_main_kb_check (kb, buffer, log_count);
         }
       if ((log_count >= attempts) && (attempts != 0))
         {
@@ -1962,21 +1980,24 @@ open_sock_tcp (struct script_infos *args, unsigned int port, int timeout)
            */
           if (host_get_port_state (args, port) > 0)
             {
-              char ip_str[INET6_ADDRSTRLEN];
+              char host_port_ip_str[INET6_ADDRSTRLEN];
 
               g_snprintf (buffer, sizeof (buffer), "Ports/tcp/%d", port);
               g_message ("open_sock_tcp: %s:%d too many timeouts. "
                          "This port will be set to closed.",
-                         ip_str, port);
-              kb_item_set_int (kb, buffer, 0);
+                         host_port_ip_str, port);
+              kb_item_set_int_with_main_kb_check (kb, buffer, 0);
 
-              addr6_to_str (args->ip, ip_str);
+              addr6_to_str (args->ip, host_port_ip_str);
               snprintf (
                 buffer, sizeof (buffer),
                 "ERRMSG|||%s|||%s|||%d/tcp||| |||Too many timeouts. The port"
                 " was set to closed.",
-                ip_str, plug_current_vhost () ?: " ", port);
-              kb_item_push_str (args->results, "internal/results", buffer);
+                host_port_ip_str,
+                plug_current_vhost () ? plug_current_vhost () : " ", port);
+
+              kb_item_push_str_with_main_kb_check (get_main_kb (),
+                                                   "internal/results", buffer);
             }
         }
       g_free (ip_str);
@@ -2038,7 +2059,6 @@ recv_line (int soc, char *buf, size_t bufsiz)
   /* Dirty SSL hack */
   if (OPENVAS_STREAM (soc))
     {
-      unsigned int ret = 0;
       buf[0] = '\0';
 
       do
