@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Based on work Copyright (C) 1998 - 2007 Tenable Network Security, Inc.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -23,8 +23,8 @@
  * @brief Header file for module network.
  */
 
-#ifndef OPENVAS_NETWORK_H
-#define OPENVAS_NETWORK_H
+#ifndef MISC_NETWORK_H
+#define MISC_NETWORK_H
 
 #include "scanneraux.h"
 
@@ -84,6 +84,9 @@ get_sock_infos (int sock, int *r_transport, void **r_tls_session);
 unsigned short *
 getpts (char *, int *);
 
+void
+open_stream_tls_default_priorities (const char *p, const int pflag);
+
 int
 open_stream_connection (struct script_infos *, unsigned int, int, int);
 
