@@ -1,4 +1,4 @@
-/* Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Copyright (C) 2009-2022 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -16,8 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef MISC2_STRUTILS_H__
-#define MISC2_STRUTILS_H__
+#ifndef MISC_STRUTILS_H
+#define MISC_STRUTILS_H
 
 #include <glib.h>
 
