@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Based on work Copyright (C) 2002 - 2004 Tenable Network Security
  *
  * SPDX-License-Identifier: GPL-2.0-only
@@ -22,6 +22,7 @@
 #include "exec.h"
 
 #include "../misc/plugutils.h"
+#include "lint.h"
 #include "nasl.h"
 #include "nasl_debug.h"
 #include "nasl_func.h"
@@ -31,6 +32,7 @@
 #include "nasl_tree.h"
 #include "nasl_var.h"
 
+#include <errno.h>       /* for errno */
 #include <glib.h>        /* for g_get_current_dir and others */
 #include <glib/gstdio.h> /* for g_chdir */
 #include <gvm/base/logging.h>
@@ -49,7 +51,7 @@
 #define G_LOG_DOMAIN "lib  nasl"
 
 extern int
-naslparse (naslctxt *);
+naslparse (naslctxt *, int *);
 
 static int
 cell2bool (lex_ctxt *lexic, tree_cell *c)
@@ -1379,11 +1381,14 @@ nasl_exec (lex_ctxt *lexic, tree_cell *st)
               len2 = (s2 == NULL ? 0 : strlen (s2));
             }
 
+          /* if p1 is null, last condition p=memem() will not be evaluated
+           * and p remains NULL */
           if (len2 == 0 || len1 < len2
               || (p1 != NULL && (p = memmem (p1, len1, p2, len2)) == NULL))
             {
               s3 = g_malloc0 (len1 + 1);
-              memcpy (s3, p1, len1);
+              if (p1 != NULL)
+                memcpy (s3, p1, len1);
               ret = alloc_typed_cell (flag);
               ret->x.str_val = s3;
               ret->size = len1;
@@ -1401,7 +1406,7 @@ nasl_exec (lex_ctxt *lexic, tree_cell *st)
                   s3 = g_malloc0 (sz + 1);
                   if (p - p1 > 0)
                     memcpy (s3, p1, p - p1);
-                  if (sz > p - p1)
+                  if (p != NULL && sz > p - p1)
                     memcpy (s3 + (p - p1), p + len2, sz - (p - p1));
                 }
               ret = alloc_typed_cell (flag);
@@ -1603,9 +1608,6 @@ nasl_exec (lex_ctxt *lexic, tree_cell *st)
     }
 }
 
-extern tree_cell *
-nasl_lint (lex_ctxt *, tree_cell *);
-
 /**
  * @brief Execute a NASL script.
  *
@@ -1626,7 +1628,7 @@ exec_nasl_script (struct script_infos *script_infos, int mode)
 {
   naslctxt ctx;
   nasl_func *pf;
-  int err = 0, to, process_id;
+  int err = 0, to;
   tree_cell *ret;
   lex_ctxt *lexic;
   gchar *old_dir;
@@ -1634,6 +1636,7 @@ exec_nasl_script (struct script_infos *script_infos, int mode)
   tree_cell tc;
   const char *str, *name = script_infos->name, *oid = script_infos->oid;
   gchar *short_name = g_path_get_basename (name);
+  int error_counter = 0;
 
   nasl_set_plugin_filename (short_name);
   g_free (short_name);
@@ -1646,8 +1649,8 @@ exec_nasl_script (struct script_infos *script_infos, int mode)
 
   if (g_chdir (newdir) != 0)
     {
-      g_message ("%s: Not able to open nor to locate it in include paths",
-                 name);
+      g_message ("%s: Not able to change working directory to %s (%d [%s]).",
+                 __func__, newdir, errno, strerror (errno));
       g_free (old_dir);
       g_free (newdir);
       return -1;
@@ -1666,9 +1669,10 @@ exec_nasl_script (struct script_infos *script_infos, int mode)
 
   if (init_nasl_ctx (&ctx, name) == 0)
     {
-      if (naslparse (&ctx))
+      err = naslparse (&ctx, &error_counter);
+      if (err != 0 || error_counter > 0)
         {
-          g_message ("%s: Parse error at or near line %d", name, ctx.line_nb);
+          g_message ("%s. There were %d parse errors.", name, error_counter);
           nasl_clean_ctx (&ctx);
           g_chdir (old_dir);
           g_free (old_dir);
@@ -1698,20 +1702,19 @@ exec_nasl_script (struct script_infos *script_infos, int mode)
 
   lexic->recv_timeout = to;
 
-  process_id = getpid ();
   if (mode & NASL_LINT)
     {
       /* ret is set to the number of errors the linter finds.
       ret will be overwritten with -1 if any errors occur in the steps
       after linting so we do not break other behaviour dependent on a
       negative return value when doing more than just linting. */
-      tree_cell *ret = nasl_lint (lexic, ctx.tree);
-      if (ret == NULL)
+      tree_cell *lintret = nasl_lint (lexic, ctx.tree);
+      if (lintret == NULL)
         err--;
-      else if (ret != FAKE_CELL && ret->x.i_val > 0)
+      else if (lintret != FAKE_CELL && lintret->x.i_val > 0)
         {
-          err = ret->x.i_val;
-          g_free (ret);
+          err = lintret->x.i_val;
+          g_free (lintret);
         }
     }
   else if (!(mode & NASL_EXEC_PARSE_ONLY))
@@ -1757,8 +1760,5 @@ exec_nasl_script (struct script_infos *script_infos, int mode)
 
   nasl_clean_ctx (&ctx);
   free_lex_ctxt (lexic);
-  if (process_id != getpid ())
-    exit (0);
-
   return err;
 }
