@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Based on work Copyright (C) 2002 Renaud Deraison
  *
  * SPDX-License-Identifier: GPL-2.0-only
@@ -19,13 +19,16 @@
 
 #define SMART_TCP_RW
 
+#include "../misc/kb_cache.h"  /* for get_main_kb */
 #include "../misc/network.h"   /* for get_encaps_through */
 #include "../misc/plugutils.h" /* for OPENVAS_ENCAPS_IP */
+#include "nasl_builtin_plugins.h"
 #include "nasl_lex_ctxt.h"
 
 #include <ctype.h> /* for tolower() */
 #include <errno.h> /* for errno() */
 #include <glib.h>
+#include <gvm/util/mqtt.h>
 #include <gvm/util/nvticache.h>
 #include <regex.h>     /* for regex_t */
 #include <signal.h>    /* for signal() */
@@ -48,6 +51,10 @@
 
 #define NUM_CHILDREN "Number of connections done in parallel : "
 
+// we cannot use the GNU ones due to number mismatch
+#define TLS_PRIME_UNACCEPTABLE -2
+#define TLS_FATAL_ALERT -3
+
 #undef G_LOG_DOMAIN
 /**
  * @brief GLib logging domain.
@@ -112,14 +119,14 @@ mark_chargen_server (struct script_infos *desc, int port)
   post_log (oid, desc, port, "Chargen is running on this port");
 }
 
-void
+static void
 mark_echo_server (struct script_infos *desc, int port)
 {
   register_service (desc, port, "echo");
   post_log (oid, desc, port, "An echo server is running on this port");
 }
 
-void
+static void
 mark_ncacn_http_server (struct script_infos *desc, int port, char *buffer)
 {
   char ban[256];
@@ -137,7 +144,7 @@ mark_ncacn_http_server (struct script_infos *desc, int port, char *buffer)
     }
 }
 
-void
+static void
 mark_vnc_server (struct script_infos *desc, int port, char *buffer)
 {
   char ban[512];
@@ -146,7 +153,7 @@ mark_vnc_server (struct script_infos *desc, int port, char *buffer)
   plug_replace_key (desc, ban, ARG_STRING, buffer);
 }
 
-void
+static void
 mark_nntp_server (struct script_infos *desc, int port, char *buffer, int trp)
 {
   char ban[512];
@@ -158,19 +165,19 @@ mark_nntp_server (struct script_infos *desc, int port, char *buffer, int trp)
   post_log (oid, desc, port, ban);
 }
 
-void
+static void
 mark_swat_server (struct script_infos *desc, int port)
 {
   register_service (desc, port, "swat");
 }
 
-void
+static void
 mark_vqserver (struct script_infos *desc, int port)
 {
   register_service (desc, port, "vqServer-admin");
 }
 
-void
+static void
 mark_mldonkey (struct script_infos *desc, int port)
 {
   char ban[512];
@@ -179,7 +186,7 @@ mark_mldonkey (struct script_infos *desc, int port)
   post_log (oid, desc, port, ban);
 }
 
-void
+static void
 mark_http_server (struct script_infos *desc, int port, unsigned char *buffer,
                   int trp)
 {
@@ -192,7 +199,7 @@ mark_http_server (struct script_infos *desc, int port, unsigned char *buffer,
   post_log (oid, desc, port, ban);
 }
 
-void
+static void
 mark_locked_adsubtract_server (struct script_infos *desc, int port,
                                unsigned char *buffer, int trp)
 {
@@ -213,7 +220,7 @@ mark_gopher_server (struct script_infos *desc, int port)
   post_log (oid, desc, port, "A gopher server is running on this port");
 }
 
-void
+static void
 mark_rmserver (struct script_infos *desc, int port, char *buffer, int trp)
 {
   char ban[512];
@@ -226,7 +233,7 @@ mark_rmserver (struct script_infos *desc, int port, char *buffer, int trp)
   post_log (oid, desc, port, ban);
 }
 
-void
+static void
 mark_smtp_server (struct script_infos *desc, int port, char *buffer, int trp)
 {
   char ban[512];
@@ -251,7 +258,7 @@ Here is its banner : \n%s",
   }
 }
 
-void
+static void
 mark_snpp_server (struct script_infos *desc, int port, char *buffer, int trp)
 {
   char ban[512], *report, *t;
@@ -271,7 +278,7 @@ Here is its banner : \n%s",
   g_free (report);
 }
 
-void
+static void
 mark_ftp_server (struct script_infos *desc, int port, char *buffer, int trp)
 {
   register_service (desc, port, "ftp");
@@ -306,7 +313,7 @@ Here is its banner : \n%s",
     }
 }
 
-void
+static void
 mark_ssh_server (struct script_infos *desc, int port, char *buffer)
 {
   register_service (desc, port, "ssh");
@@ -316,7 +323,7 @@ mark_ssh_server (struct script_infos *desc, int port, char *buffer)
   post_log (oid, desc, port, "An ssh server is running on this port");
 }
 
-void
+static void
 mark_http_proxy (struct script_infos *desc, int port, int trp)
 {
   char ban[512];
@@ -327,7 +334,7 @@ mark_http_proxy (struct script_infos *desc, int port, int trp)
   post_log (oid, desc, port, ban);
 }
 
-void
+static void
 mark_pop_server (struct script_infos *desc, int port, char *buffer)
 {
   char *c = strchr (buffer, '\n');
@@ -363,7 +370,7 @@ mark_pop_server (struct script_infos *desc, int port, char *buffer)
   g_free (buffer2);
 }
 
-void
+static void
 mark_imap_server (struct script_infos *desc, int port, char *buffer, int trp)
 {
   char ban[512];
@@ -377,7 +384,7 @@ mark_imap_server (struct script_infos *desc, int port, char *buffer, int trp)
   }
 }
 
-void
+static void
 mark_auth_server (struct script_infos *desc, int port)
 {
   register_service (desc, port, "auth");
@@ -388,7 +395,7 @@ mark_auth_server (struct script_infos *desc, int port)
  * Postgres, MySQL & CVS pserver detection by Vincent Renardias
  * <vincent@strongholdnet.com>
  */
-void
+static void
 mark_postgresql (struct script_infos *desc, int port)
 {
   register_service (desc, port, "postgresql");
@@ -396,7 +403,7 @@ mark_postgresql (struct script_infos *desc, int port)
   post_log (oid, desc, port, "A PostgreSQL server is running on this port");
 }
 
-void
+static void
 mark_sphinxql (struct script_infos *desc, int port)
 {
   register_service (desc, port, "sphinxql");
@@ -405,7 +412,7 @@ mark_sphinxql (struct script_infos *desc, int port)
             "seems to be running on this port");
 }
 
-void
+static void
 mark_mysql (struct script_infos *desc, int port)
 {
   register_service (desc, port, "mysql");
@@ -413,7 +420,7 @@ mark_mysql (struct script_infos *desc, int port)
   post_log (oid, desc, port, "A MySQL server is running on this port");
 }
 
-void
+static void
 mark_cvspserver (struct script_infos *desc, int port)
 {
   register_service (desc, port, "cvspserver");
@@ -421,14 +428,14 @@ mark_cvspserver (struct script_infos *desc, int port)
   post_log (oid, desc, port, "A CVS pserver server is running on this port");
 }
 
-void
+static void
 mark_cvsupserver (struct script_infos *desc, int port)
 {
   register_service (desc, port, "cvsup");
   post_log (oid, desc, port, "A CVSup server is running on this port");
 }
 
-void
+static void
 mark_cvslockserver (struct script_infos *desc, int port)
 {
   register_service (desc, port, "cvslockserver");
@@ -436,14 +443,14 @@ mark_cvslockserver (struct script_infos *desc, int port)
   post_log (oid, desc, port, "A CVSLock server server is running on this port");
 }
 
-void
+static void
 mark_rsync (struct script_infos *desc, int port)
 {
   register_service (desc, port, "rsync");
   post_log (oid, desc, port, "A rsync server is running on this port");
 }
 
-void
+static void
 mark_wild_shell (struct script_infos *desc, int port)
 {
   register_service (desc, port, "wild_shell");
@@ -454,7 +461,7 @@ mark_wild_shell (struct script_infos *desc, int port)
     NULL);
 }
 
-void
+static void
 mark_telnet_server (struct script_infos *desc, int port, int trp)
 {
   char ban[255];
@@ -467,7 +474,7 @@ mark_telnet_server (struct script_infos *desc, int port, int trp)
   }
 }
 
-void
+static void
 mark_gnome14_server (struct script_infos *desc, int port, int trp)
 {
   char ban[255];
@@ -480,7 +487,7 @@ mark_gnome14_server (struct script_infos *desc, int port, int trp)
   }
 }
 
-void
+static void
 mark_eggdrop_server (struct script_infos *desc, int port, int trp)
 {
   char ban[255];
@@ -494,14 +501,14 @@ mark_eggdrop_server (struct script_infos *desc, int port, int trp)
   }
 }
 
-void
+static void
 mark_netbus_server (struct script_infos *desc, int port)
 {
   register_service (desc, port, "netbus");
   post_alarm (oid, desc, port, "NetBus is running on this port", NULL);
 }
 
-void
+static void
 mark_linuxconf (struct script_infos *desc, int port, unsigned char *buffer)
 {
   char ban[512];
@@ -926,7 +933,7 @@ mark_nagiosd_server (struct script_infos *desc, int port, int trp)
 }
 
 /*
- * Sig from  Michael Lffler <nimrod@n1mrod.de>
+ * Sig from  Michael Löffler <nimrod@n1mrod.de>
  *
  * 00: 5b 54 53 5d 0a 65 72 72 6f 72 0a                   [TS].error.
  *
@@ -1086,7 +1093,7 @@ mark_stonegate_auth_server (struct script_infos *desc, int port, int trp)
   post_log (oid, desc, port, ban);
 }
 
-void
+static void
 mark_listserv_server (struct script_infos *desc, int port, int trp)
 {
   char ban[255];
@@ -1099,7 +1106,7 @@ mark_listserv_server (struct script_infos *desc, int port, int trp)
   }
 }
 
-void
+static void
 mark_fssniffer (struct script_infos *desc, int port, int trp)
 {
   char ban[255];
@@ -1112,7 +1119,7 @@ mark_fssniffer (struct script_infos *desc, int port, int trp)
   }
 }
 
-void
+static void
 mark_remote_nc_server (struct script_infos *desc, int port, int trp)
 {
   char ban[255];
@@ -1486,6 +1493,64 @@ may_be_time (time_t *rtime)
     return 0;
 }
 
+static int
+retry_stream_connection (int test_ssl, struct script_infos *desc, int port,
+                         int timeout, int *trp)
+{
+  const char *p = "NORMAL:+ARCFOUR-128:%COMPAT";
+  const char *lp = "LEGACY:%COMPAT:%UNSAFE_RENEGOTIATION";
+  int cnx;
+
+  if (test_ssl)
+    *trp = OPENVAS_ENCAPS_TLScustom;
+  else
+    *trp = OPENVAS_ENCAPS_IP;
+
+  cnx = open_stream_connection (desc, port, *trp, timeout);
+  if (test_ssl)
+    {
+      switch (cnx)
+        {
+        case TLS_PRIME_UNACCEPTABLE:
+          // retry with insecure bit
+          g_debug ("%s: NO_PRIORITY_FLAGS failed, retrying with "
+                   "INSECURE_DH_PRIME_BITS",
+                   __func__);
+          cnx = open_stream_connection_ext (desc, port, *trp, timeout, p,
+                                            INSECURE_DH_PRIME_BITS);
+          if (cnx >= 0)
+            {
+              open_stream_tls_default_priorities (p, INSECURE_DH_PRIME_BITS);
+            }
+          break;
+        case TLS_FATAL_ALERT:
+          // retry with legacy option
+          g_debug ("%s: %s failed, retrying with %s", __func__, p, lp);
+          cnx = open_stream_connection_ext (desc, port, *trp, timeout, lp,
+                                            NO_PRIORITY_FLAGS);
+          if (cnx >= 0)
+            {
+              open_stream_tls_default_priorities (lp, NO_PRIORITY_FLAGS);
+            }
+          break;
+        default:
+          // do nothing
+          break;
+        }
+      // verify if retries went successful and if not retry without tls
+      if (cnx < 0)
+        {
+          g_debug ("%s: unable to establish a TLS connection to %s; falling "
+                   "back to unencrypted connection",
+                   __func__, plug_get_host_fqdn (desc));
+          *trp = OPENVAS_ENCAPS_IP;
+          cnx = open_stream_connection (desc, port, *trp, timeout);
+        }
+    }
+
+  return cnx;
+}
+
 static int
 plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
 {
@@ -1528,7 +1593,7 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
           char buffer[2049];
           unsigned char *banner = NULL, *bannerHex = NULL;
           size_t banner_len, i;
-          int port = atoi (h->data + len_head);
+          int port = atoi ((const char *) h->data + len_head);
           int flg = 0;
           int unindentified_service = 0;
           int three_digits = 0;
@@ -1549,13 +1614,13 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
           if (type == ARG_STRING && bannerHex != NULL && bannerHex[0] != '\0')
             {
               int c1, c2;
-              unsigned int i;
+              unsigned int j;
               banner_len = strlen ((char *) bannerHex) / 2;
               if (banner_len >= sizeof (buffer))
                 banner_len = sizeof (buffer) - 1;
-              for (i = 0; i < banner_len; i++)
+              for (j = 0; j < banner_len; j++)
                 {
-                  c1 = bannerHex[2 * i];
+                  c1 = bannerHex[2 * j];
                   if (c1 >= 0 && c1 <= 9)
                     c1 -= '0';
                   else if (c1 >= 'a' && c1 <= 'f')
@@ -1564,7 +1629,7 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
                     c1 -= 'A';
                   else
                     banner_len = 0; /* Invalid value */
-                  c2 = bannerHex[2 * i + 1];
+                  c2 = bannerHex[2 * j + 1];
                   if (c2 >= 0 && c2 <= 9)
                     c2 -= '0';
                   else if (c2 >= 'a' && c2 <= 'f')
@@ -1573,9 +1638,9 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
                     c2 -= 'A';
                   else
                     banner_len = 0; /* Invalid value */
-                  buffer[i] = c1 << 4 | c2;
+                  buffer[j] = c1 << 4 | c2;
                 }
-              buffer[i] = '\0';
+              buffer[j] = '\0';
               if (banner_len > 0)
                 banner = (unsigned char *) buffer;
             }
@@ -1599,36 +1664,9 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
                   g_free (banner);
                   banner = NULL;
                 }
-              /* If test_ssl is set, try with TLS first. */
-              if (test_ssl)
-                trp = OPENVAS_ENCAPS_TLScustom;
-              else
-                trp = OPENVAS_ENCAPS_IP;
               gettimeofday (&tv1, NULL);
-              cnx = open_stream_connection (desc, port, trp, cnx_timeout);
-              if (cnx == -2 && test_ssl)
-                {
-                  unsigned int flags = INSECURE_DH_PRIME_BITS;
-
-                  gettimeofday (&tv1, NULL);
-                  cnx = open_stream_connection_ext (
-                    desc, port, trp, cnx_timeout, "NORMAL:+ARCFOUR-128:%COMPAT",
-                    flags);
-                }
-              else if (cnx < 0 && test_ssl)
-                {
-                  if (cnx == -3)
-                    {
-                      host_fqdn = plug_get_host_fqdn (desc);
-                      g_message ("%s: A TLS fatal alert has been received "
-                                 "during the handshake with %s:%d",
-                                 __func__, host_fqdn, port);
-                      g_free (host_fqdn);
-                    }
-                  trp = OPENVAS_ENCAPS_IP;
-                  gettimeofday (&tv1, NULL);
-                  cnx = open_stream_connection (desc, port, trp, cnx_timeout);
-                }
+              cnx = retry_stream_connection (test_ssl, desc, port, cnx_timeout,
+                                             &trp);
               gettimeofday (&tv2, NULL);
               diff_tv = DIFFTV1000 (tv2, tv1);
             }
@@ -2043,6 +2081,7 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
                                && strstr (
                                  buffer,
                                  "That item is not currently available")))
+
                     mark_gopher_server (desc, port);
                   else if (strstr (buffer,
                                    "www-authenticate: basic realm=\"swat\""))
@@ -2284,7 +2323,7 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
                                  * OPENVAS_ENCAPS_IP &&
                                  wrap_timeout > 0 */ )
                 {
-                  int nfd, fd, x, flag = 0;
+                  int nfd, fd, wx, flag = 0;
                   char b;
 
                   nfd = open_stream_connection (desc, port, OPENVAS_ENCAPS_IP,
@@ -2301,20 +2340,20 @@ plugin_do_run (struct script_infos *desc, GSList *h, int test_ssl)
                       signal (SIGALRM, SIG_IGN);
 
                       (void) gettimeofday (&tv1, NULL);
-                      x = select (fd + 1, &rfds, NULL, NULL, &tv);
+                      wx = select (fd + 1, &rfds, NULL, NULL, &tv);
                       (void) gettimeofday (&tv2, NULL);
                       diff_tv2 = DIFFTV1000 (tv2, tv1);
-                      if (x < 0)
+                      if (wx < 0)
                         {
                           if (errno == EINTR)
                             goto select_again2;
                           perror ("select");
                         }
-                      else if (x > 0)
+                      else if (wx > 0)
                         {
                           errno = 0;
-                          x = recv (fd, &b, 1, MSG_DONTWAIT);
-                          if (x == 0 || (x < 0 && errno == EPIPE))
+                          wx = recv (fd, &b, 1, MSG_DONTWAIT);
+                          if (wx == 0 || (wx < 0 && errno == EPIPE))
                             {
                               /*
                                * If the service quickly closes the connection
@@ -2545,10 +2584,13 @@ plugin_run_find_service (lex_ctxt *lexic)
           if (sons[i] == 0)
             {
               kb_lnk_reset (kb);
+              kb_lnk_reset (get_main_kb ());
+              mqtt_reset ();
               nvticache_reset ();
+
               signal (SIGTERM, _exit);
               plugin_do_run (desc, sons_args[i], test_ssl);
-              exit (0);
+              _exit (0);
             }
           else
             {
