@@ -1,4 +1,4 @@
-/* Copyright (C) 2010-2021 Greenbone Networks GmbH
+/* Copyright (C) 2010-2022 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -22,9 +22,10 @@
  * @brief Header file for built-in plugins.
  */
 
-#ifndef _NASL_BUILTIN_PLUGINS_H
-#define _NASL_BUILTIN_PLUGINS_H
-
+#ifndef NASL_NASL_BUILTIN_PLUGINS_H
+#define NASL_NASL_BUILTIN_PLUGINS_H
+#include "nasl_lex_ctxt.h"
+#include "nasl_tree.h"
 tree_cell *
 plugin_run_find_service (lex_ctxt *);
 
@@ -34,4 +35,4 @@ plugin_run_openvas_tcp_scanner (lex_ctxt *);
 tree_cell *
 plugin_run_synscan (lex_ctxt *);
 
-#endif /* not _NASL_BUILTIN_PLUGINS_H */
+#endif /* not NASL_NASL_BUILTIN_PLUGINS_H */
