@@ -1,4 +1,4 @@
-/* Copyright (C) 2012-2021 Greenbone Networks GmbH
+/* Copyright (C) 2012-2022 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -24,8 +24,8 @@
  * This file contains the protos for \ref nasl_cert.c
  */
 
-#ifndef NASL_CERT_H
-#define NASL_CERT_H
+#ifndef NASL_NASL_CERT_H
+#define NASL_NASL_CERT_H
 
 #include "nasl_lex_ctxt.h"
 
@@ -38,4 +38,4 @@ nasl_cert_close (lex_ctxt *lexic);
 tree_cell *
 nasl_cert_query (lex_ctxt *lexic);
 
-#endif /*NASL_CERT_H*/
+#endif /*NASL_NASL_CERT_H*/
