@@ -16,21 +16,25 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef _NASL_CTX_H
-#define _NASL_CTX_H
+#ifndef NASL_NASL_GLOBAL_CTX_H
+#define NASL_NASL_GLOBAL_CTX_H
 
 /* for FILE */
+#include "nasl_tree.h"
+
 #include <gvm/util/kb.h>
 #include <stdio.h>
 
 typedef struct
 {
   int line_nb;
+  char *name;
   int always_signed; /**< If set disable signature check during scans and feed
                         upload. */
   int exec_descr; /**< Tell grammar that is a feed upload process or a running a
                      scan process. */
   int index;
+  unsigned int include_order;
   tree_cell *tree;
   char *buffer;
   kb_t kb;
