@@ -16,6 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "nasl_init.h"
+
 #include "../misc/network.h"        /* for OPENVAS_ENCAPS_* */
 #include "../misc/nvt_categories.h" /* for ACT_INIT */
 #include "exec.h"
@@ -84,7 +86,6 @@ static init_func libfuncs[] = {
   {"script_get_preference_file_location", script_get_preference_file_location},
   {"script_oid", script_oid},
   {"script_cve_id", script_cve_id},
-  {"script_bugtraq_id", script_bugtraq_id},
   {"script_xref", script_xref},
   {"script_tag", script_tag},
   {"vendor_version", nasl_vendor_version},
@@ -107,6 +108,7 @@ static init_func libfuncs[] = {
   {"recv", nasl_recv},
   {"recv_line", nasl_recv_line},
   {"send", nasl_send},
+  {"get_mtu", nasl_get_mtu},
   {"socket_negotiate_ssl", nasl_socket_negotiate_ssl},
   {"socket_check_ssl_safe_renegotiation",
    nasl_socket_check_ssl_safe_renegotiation},
@@ -261,6 +263,7 @@ static init_func libfuncs[] = {
   {"MD5", nasl_md5},
   {"SHA1", nasl_sha1},
   {"SHA256", nasl_sha256},
+  {"SHA512", nasl_sha512},
   {"RIPEMD160", nasl_ripemd160},
   {"HMAC_MD2", nasl_hmac_md2},
   {"HMAC_MD5", nasl_hmac_md5},
@@ -285,6 +288,8 @@ static init_func libfuncs[] = {
   {"dec2str", nasl_dec2str},
   {"get_signature", nasl_get_sign},
   {"get_smb2_signature", nasl_get_smb2_sign},
+  {"smb_cmac_aes_signature", nasl_smb_cmac_aes_sign},
+  {"smb_gmac_aes_signature", nasl_smb_gmac_aes_sign},
   {"dh_generate_key", nasl_dh_generate_key},
   {"bn_random", nasl_bn_random},
   {"bn_cmp", nasl_bn_cmp},
@@ -295,12 +300,29 @@ static init_func libfuncs[] = {
   {"bf_cbc_encrypt", nasl_bf_cbc_encrypt},
   {"bf_cbc_decrypt", nasl_bf_cbc_decrypt},
   {"rc4_encrypt", nasl_rc4_encrypt},
+  {"aes_mac_cbc", nasl_aes_mac_cbc},
+  {"aes_mac_gcm", nasl_aes_mac_gcm},
   {"aes128_cbc_encrypt", nasl_aes128_cbc_encrypt},
   {"aes256_cbc_encrypt", nasl_aes256_cbc_encrypt},
   {"aes128_ctr_encrypt", nasl_aes128_ctr_encrypt},
   {"aes256_ctr_encrypt", nasl_aes256_ctr_encrypt},
   {"aes128_gcm_encrypt", nasl_aes128_gcm_encrypt},
+  {"aes128_gcm_encrypt_auth", nasl_aes128_gcm_encrypt_auth},
+  {"aes128_gcm_decrypt", nasl_aes128_gcm_decrypt},
+  {"aes128_gcm_decrypt_auth", nasl_aes128_gcm_decrypt_auth},
   {"aes256_gcm_encrypt", nasl_aes256_gcm_encrypt},
+  {"aes256_gcm_encrypt_auth", nasl_aes256_gcm_encrypt_auth},
+  {"aes256_gcm_decrypt", nasl_aes256_gcm_decrypt},
+  {"aes256_gcm_decrypt_auth", nasl_aes256_gcm_decrypt_auth},
+  {"aes128_ccm_encrypt", nasl_aes128_ccm_encrypt},
+  {"aes128_ccm_encrypt_auth", nasl_aes128_ccm_encrypt_auth},
+  {"aes128_ccm_decrypt", nasl_aes128_ccm_decrypt},
+  {"aes128_ccm_decrypt_auth", nasl_aes128_ccm_decrypt_auth},
+  {"aes256_ccm_encrypt", nasl_aes256_ccm_encrypt},
+  {"aes256_ccm_encrypt_auth", nasl_aes256_ccm_encrypt_auth},
+  {"aes256_ccm_decrypt", nasl_aes256_ccm_decrypt},
+  {"aes256_ccm_decrypt_auth", nasl_aes256_ccm_decrypt_auth},
+  {"smb3kdf", nasl_smb3kdf},
   {"des_ede_cbc_encrypt", nasl_des_ede_cbc_encrypt},
   {"open_rc4_cipher", nasl_open_rc4_cipher},
   {"close_stream_cipher", nasl_close_stream_cipher},
@@ -313,8 +335,11 @@ static init_func libfuncs[] = {
   {"gzip", nasl_gzip},
   {"DES", nasl_cipher_des},
   {"snmpv1_get", nasl_snmpv1_get},
+  {"snmpv1_getnext", nasl_snmpv1_getnext},
   {"snmpv2c_get", nasl_snmpv2c_get},
+  {"snmpv2c_getnext", nasl_snmpv2c_getnext},
   {"snmpv3_get", nasl_snmpv3_get},
+  {"snmpv3_getnext", nasl_snmpv3_getnext},
   {"ssh_connect", nasl_ssh_connect},
   {"ssh_disconnect", nasl_ssh_disconnect},
   {"ssh_session_id_from_sock", nasl_ssh_session_id_from_sock},
