@@ -81,7 +81,9 @@
  * Checksum routine for Internet Protocol family headers (C Version)
  * From ping examples in W.Richard Stevens "UNIX NETWORK PROGRAMMING" book.
  */
-static int np_in_cksum (p, n) u_short *p;
+static int
+np_in_cksum (p, n)
+u_short *p;
 int n;
 {
   register u_short answer = 0;
@@ -2159,8 +2161,8 @@ forge_igmp_v6_packet (lex_ctxt *lexic)
       igmp->cksum = np_in_cksum ((u_short *) igmp, sizeof (struct igmp6_hdr));
       if (data != NULL)
         {
-          char *p = (char *) (pkt + 40 + sizeof (struct igmp6_hdr));
-          bcopy (p, data, len);
+          char *ptmp = (char *) (pkt + 40 + sizeof (struct igmp6_hdr));
+          bcopy (ptmp, data, len);
         }
       retc = alloc_typed_cell (CONST_DATA);
       retc->x.str_val = (char *) pkt;
