@@ -83,16 +83,33 @@ isalldigit (char *str, int len)
  * scanner.
  */
 
+/**
+ * @brief Add timeout preference to VT preferences
+ *
+ * VT timeout is handled as normal VT preference.
+ * Because of backward compatibility issues the timeout preference is always
+ * located at the VT pref location with id NVTPREF_TIMEOUT_ID.
+ *
+ * @param[in] lexic   lexic
+ * @param[in] to      script timeout
+ *
+ * @return FAKE_CELL
+ */
 tree_cell *
 script_timeout (lex_ctxt *lexic)
 {
   nvti_t *nvti = lexic->script_infos->nvti;
   int to = get_int_var_by_num (lexic, 0, -65535);
+  nvtpref_t *np;
+  gchar *timeout;
 
   if (to == -65535)
     return FAKE_CELL;
 
-  nvti_set_timeout (nvti, to ? to : -1);
+  timeout = g_strdup_printf ("%d", to);
+
+  np = nvtpref_new (NVTPREF_TIMEOUT_ID, "timeout", "entry", timeout);
+  nvti_add_pref (nvti, np);
   return FAKE_CELL;
 }
 
@@ -119,22 +136,6 @@ script_cve_id (lex_ctxt *lexic)
   return FAKE_CELL;
 }
 
-tree_cell *
-script_bugtraq_id (lex_ctxt *lexic)
-{
-  struct script_infos *script_infos = lexic->script_infos;
-  char *bid = get_str_var_by_num (lexic, 0);
-  int i;
-
-  for (i = 0; bid != NULL; i++)
-    {
-      nvti_add_vtref (script_infos->nvti, vtref_new ("bid", bid, ""));
-      bid = get_str_var_by_num (lexic, i + 1);
-    }
-
-  return FAKE_CELL;
-}
-
 /**
  * @brief Add a cross reference to the meta data.
  *
@@ -144,18 +145,12 @@ script_bugtraq_id (lex_ctxt *lexic)
  * Alternative to "value", "csv" can be used with a
  * list of comma-separated values.
  *
- * In fact, if name is "cve" or "bid", it is equivalent
- * to call script_cve_id() or script_bugtraq_id(), for example
+ * In fact, if name is "cve", it is equivalent
+ * to call script_cve_id(), for example
  * script_cve_id ("CVE-2019-12345");
  * is identical to
  * script_xref (name: "cve", value: "CVE-2019-12345");
  *
- * And also:
- * script_bugtraq_id (12345);
- * is identical to
- * script_xref (name: "bid", value: "12345");
- * (watch out that the number now needs to be a string).
- *
  * This even works with multiple comma-separated elements like
  * script_xref (name: "cve", csv: "CVE-2019-12345,CVE-2019-54321");
  *
@@ -327,7 +322,8 @@ script_require_keys (lex_ctxt *lexic)
   if (keys == NULL)
     {
       nasl_perror (lexic, "Argument error in function script_require_keys()\n");
-      nasl_perror (lexic, "Function usage is : script_require_keys(<name>)\n");
+      nasl_perror (lexic,
+                   "Function usage is : script_require_keys(<name>...)\n");
       nasl_perror (lexic, "Where <name> is the name of a key\n");
       return FAKE_CELL;
     }
@@ -352,9 +348,10 @@ script_mandatory_keys (lex_ctxt *lexic)
     {
       nasl_perror (lexic,
                    "Argument error in function script_mandatory_keys()\n");
-      nasl_perror (lexic,
-                   "Function usage is : script_mandatory_keys(<name>)\n");
-      nasl_perror (lexic, "Where <name> is the name of a key\n");
+      nasl_perror (lexic, "Function usage is: script_mandatory_keys(<name>... "
+                          "[, re: '<name>=<regex>'])\n");
+      nasl_perror (lexic, "Where <name> is the name of a key and <regex> is a "
+                          "regular expression for a value of a key.\n");
       return FAKE_CELL;
     }
 
@@ -833,12 +830,84 @@ replace_kb_item (lex_ctxt *lexic)
   return FAKE_CELL;
 }
 
+/**
+ * @brief Set a volate kb item.
+ *
+ * @param[in]  lexic  NASL lexer.
+ * @param[in]  name Name of Item.
+ * @param[in]  value Value of Item.
+ * @param[in]  expire Optional expire for item in seconds.
+ *
+ * @return FAKE_CELL
+ */
+static tree_cell *
+set_kb_item_volatile (lex_ctxt *lexic)
+{
+  struct script_infos *script_infos = lexic->script_infos;
+  char *name = get_str_var_by_name (lexic, "name");
+  int type = get_var_type_by_name (lexic, "value");
+  int expire = get_int_var_by_name (lexic, "expire", -1);
+
+  if (name == NULL)
+    {
+      nasl_perror (lexic, "Syntax error with set_kb_item() [null name]\n",
+                   name);
+      return FAKE_CELL;
+    }
+
+  if (type == VAR2_INT)
+    {
+      int value = get_int_var_by_name (lexic, "value", -1);
+      if (value != -1 && expire != -1)
+        plug_set_key_volatile (script_infos, name, ARG_INT,
+                               GSIZE_TO_POINTER (value), expire);
+      else
+        nasl_perror (lexic,
+                     "Syntax error with set_kb_item() [value=-1 or expire=-1 "
+                     "for name '%s']\n",
+                     name);
+    }
+  else
+    {
+      char *value = get_str_var_by_name (lexic, "value");
+      int len = get_var_size_by_name (lexic, "value");
+      if (value == NULL || expire == -1)
+        {
+          nasl_perror (lexic,
+                       "Syntax error with set_kb_item() [null value or "
+                       "expire=-1 for name '%s']\n",
+                       name);
+          return FAKE_CELL;
+        }
+      plug_set_key_len_volatile (script_infos, name, ARG_STRING, value, expire,
+                                 len);
+    }
+
+  return FAKE_CELL;
+}
+
+/**
+ * @brief Set a kb item.
+ *
+ * If expire is set the key will be removed after it expired.
+ *
+ * @param[in]  lexic  NASL lexer.
+ * @param[in]  name Name of Item.
+ * @param[in]  value Value of Item.
+ * @param[in]  expire Optional expire for item in seconds.
+ *
+ * @return FAKE_CELL
+ */
 tree_cell *
 set_kb_item (lex_ctxt *lexic)
 {
   struct script_infos *script_infos = lexic->script_infos;
   char *name = get_str_var_by_name (lexic, "name");
   int type = get_var_type_by_name (lexic, "value");
+  int expire = get_int_var_by_name (lexic, "expire", -1);
+
+  if (expire != -1)
+    return set_kb_item_volatile (lexic);
 
   if (name == NULL)
     {
@@ -906,10 +975,11 @@ security_something (lex_ctxt *lexic, proto_post_something_t proto_post_func,
       int len = get_var_size_by_name (lexic, "data");
       int i;
 
-      dup = g_memdup2 (data, len + 1);
+      dup = g_malloc0 ((len + 1) * sizeof (char *));
+      memcpy (dup, data, len + 1);
 
       for (i = 0; i < len; i++)
-        if (dup[i] == 0)
+        if (dup[i] == '\0')
           dup[i] = ' ';
     }
 
