@@ -1,4 +1,4 @@
-/* Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Copyright (C) 2009-2022 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -17,8 +17,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef NASL_SIGNATURE_H
-#define NASL_SIGNATURE_H
+#ifndef NASL_NASL_SIGNATURE_H
+#define NASL_NASL_SIGNATURE_H
+
+#include <stddef.h>
 
 int
 nasl_verify_signature (const char *, const char *, size_t);
