@@ -1,12 +1,11 @@
-/* Portions Copyright (C) 2012-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2012-2022 Greenbone Networks GmbH
  * Based on work Copyright (C) 2002 - 2004 Tenable Network Security
  *
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-License-Identifier: GPL-2.0-only
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * version 2 as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,9 +27,12 @@
  */
 
 /*--------------------------------------------------------------------------*/
+#include "nasl_socket.h"
+
 #include "../misc/network.h"
-#include "../misc/plugutils.h" /* for plug_get_host_ip */
-#include "../misc/support.h"   /* for the g_memdup2 workaround */
+#include "../misc/pcap_openvas.h" // for routethrough
+#include "../misc/plugutils.h"    /* for plug_get_host_ip */
+#include "../misc/support.h"      /* for the g_memdup2 workaround */
 #include "exec.h"
 #include "nasl.h"
 #include "nasl_debug.h"
@@ -48,9 +50,11 @@
 #include <gvm/base/logging.h>
 #include <gvm/base/networking.h> /* for gvm_source_set_socket */
 #include <gvm/base/prefs.h>      /* for prefs_get */
-#include <netinet/in.h>          /* for sockaddr_in */
-#include <stdlib.h>              /* for atoi() */
-#include <string.h>              /* for bzero */
+#include <net/if.h>
+#include <netinet/in.h> /* for sockaddr_in */
+#include <stdlib.h>     /* for atoi() */
+#include <string.h>     /* for bzero */
+#include <sys/ioctl.h>
 #include <sys/time.h>
 #include <unistd.h> /* for close */
 
@@ -762,7 +766,8 @@ nasl_socket_get_ssl_ciphersuite (lex_ctxt *lexic)
 tree_cell *
 nasl_recv (lex_ctxt *lexic)
 {
-  char *data;
+  char *data, *resend_data;
+  int rd_len;
   int len = get_int_var_by_name (lexic, "length", -1);
   int min_len = get_int_var_by_name (lexic, "min", -1);
   int soc = get_int_var_by_name (lexic, "socket", 0);
@@ -802,10 +807,10 @@ nasl_recv (lex_ctxt *lexic)
 
           if (select (soc + 1, &rd, NULL, NULL, &tv) > 0)
             {
-              int e;
-              e = recv (soc, data + new_len, len - new_len, 0);
+              int alen;
+              alen = recv (soc, data + new_len, len - new_len, 0);
 
-              if (e <= 0)
+              if (alen <= 0)
                 {
                   if (!new_len)
                     {
@@ -814,19 +819,17 @@ nasl_recv (lex_ctxt *lexic)
                     }
                 }
               else
-                new_len += e;
+                new_len += alen;
 
               break; /* UDP data is never fragmented */
             }
           else
             {
               /* The packet may have been lost en route - we resend it */
-              char *data;
-              int len;
 
-              data = get_udp_data (lexic->script_infos, soc, &len);
-              if (data != NULL)
-                send (soc, data, len, 0);
+              resend_data = get_udp_data (lexic->script_infos, soc, &rd_len);
+              if (resend_data != NULL)
+                send (soc, resend_data, rd_len, 0);
               tv.tv_sec = to / retries;
               tv.tv_usec = (to % retries) * 100000;
             }
@@ -918,6 +921,54 @@ nasl_recv_line (lex_ctxt *lexic)
 
 /*---------------------------------------------------------------------*/
 
+static int
+get_mtu (struct in6_addr *dst)
+{
+  char *device;
+  int r = -1;
+  struct ifreq ifr;
+  int sd;
+  if ((device = v6_routethrough (dst, NULL)) == NULL)
+    goto exit;
+  memcpy (ifr.ifr_name, device, sizeof (ifr.ifr_name));
+  if ((sd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
+    goto exit;
+  if (ioctl (sd, SIOCGIFMTU, &ifr) < 0)
+    goto cexit;
+  r = ifr.ifr_mtu;
+cexit:
+  close (sd);
+exit:
+  return r;
+}
+
+static int
+get_udp_payload_size (struct in6_addr *dst)
+{
+  int r = 0;
+  // the ip header maximum size is 60 and a UDP header contains 8 bytes
+  if ((r = get_mtu (dst) - 60 - 8) < 0)
+    return -1;
+  return r;
+}
+
+tree_cell *
+nasl_get_mtu (lex_ctxt *lexic)
+{
+  int mtu;
+  if ((mtu = get_mtu (plug_get_host_ip (lexic->script_infos))) == -1)
+    {
+      nasl_perror (
+        lexic,
+        "Unable to get MTU of used interface. get_mtu is not available.\n");
+    }
+  tree_cell *retc;
+  retc = alloc_typed_cell (CONST_INT);
+  retc->x.i_val = mtu;
+
+  return retc;
+}
+
 tree_cell *
 nasl_send (lex_ctxt *lexic)
 {
@@ -927,6 +978,7 @@ nasl_send (lex_ctxt *lexic)
   int length = get_int_var_by_name (lexic, "length", 0);
   int data_length = get_var_size_by_name (lexic, "data");
   int n;
+  int mtu = -1;
   tree_cell *retc;
   int type;
   unsigned int type_len = sizeof (type);
@@ -946,6 +998,13 @@ nasl_send (lex_ctxt *lexic)
       && getsockopt (soc, SOL_SOCKET, SO_TYPE, &type, &type_len) == 0
       && type == SOCK_DGRAM)
     {
+      if ((mtu = get_udp_payload_size (plug_get_host_ip (lexic->script_infos)))
+            > 0
+          && mtu < length)
+        nasl_perror (lexic,
+                     "data payload is larger (%d) than max udp payload (%d)\n",
+                     length, mtu);
+
       n = send (soc, data, length, option);
       add_udp_data (lexic->script_infos, soc, data, length);
     }
@@ -1510,7 +1569,8 @@ nasl_socket_cert_verify (lex_ctxt *lexic)
     }
 
   /* Init ca_list and load system CA trust list */
-  if ((ret = gnutls_x509_trust_list_init (&ca_list, ca_list_size)) < 0)
+  ret = gnutls_x509_trust_list_init (&ca_list, ca_list_size);
+  if (ret < 0)
     {
       g_free (cert);
       return NULL;
