@@ -24,8 +24,11 @@
  *----------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
-#ifndef NASL_SOCKET_H
-#define NASL_SOCKET_H
+#ifndef NASL_NASL_SOCKET_H
+#define NASL_NASL_SOCKET_H
+
+#include "nasl_lex_ctxt.h"
+#include "nasl_tree.h"
 
 tree_cell *
 nasl_open_sock_tcp (lex_ctxt *);
@@ -42,6 +45,9 @@ nasl_open_priv_sock_tcp (lex_ctxt *);
 tree_cell *
 nasl_open_priv_sock_udp (lex_ctxt *);
 
+tree_cell *
+nasl_get_mtu (lex_ctxt *);
+
 tree_cell *
 nasl_send (lex_ctxt *);
 tree_cell *
