@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -26,7 +26,8 @@
 #include "hosts.h" /* for hosts_new() */
 
 #include "../misc/network.h" /* for internal_recv */
-#include "utils.h"           /* for data_left() */
+#include "../misc/plugutils.h"
+#include "utils.h" /* for data_left() */
 
 #include <errno.h>               /* for errno() */
 #include <glib.h>                /* for g_free() */
@@ -81,7 +82,10 @@ host_set_time (kb_t kb, char *ip, char *type)
   int len;
 
   t = time (NULL);
-  timestr = g_strdup (ctime (&t));
+  char ts[26];
+  char *ts_ptr = ts;
+  ctime_r (&t, ts_ptr);
+  timestr = g_strdup (ts_ptr);
   len = strlen (timestr);
   if (timestr[len - 1] == '\n')
     timestr[len - 1] = '\0';
@@ -90,7 +94,7 @@ host_set_time (kb_t kb, char *ip, char *type)
             timestr);
   g_free (timestr);
 
-  kb_item_push_str (kb, "internal/results", log_msg);
+  kb_item_push_str_with_main_kb_check (kb, "internal/results", log_msg);
 }
 
 static void
