@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -23,8 +23,8 @@
  * @brief hosts.c header.
  */
 
-#ifndef HOSTS_H
-#define HOSTS_H
+#ifndef OPENVAS_HOSTS_H
+#define OPENVAS_HOSTS_H
 
 #include "../misc/scanneraux.h"
 
