@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -33,26 +33,32 @@
  * OpenVAS main module, runs the scanner.
  */
 
+#include "openvas.h"
+
+#include "../misc/kb_cache.h"
 #include "../misc/plugutils.h"     /* nvticache_free */
+#include "../misc/scan_id.h"       /* to manage global scan_id */
 #include "../misc/vendorversion.h" /* for vendor_version_set */
 #include "attack.h"                /* for attack_network */
+#include "debug_utils.h"           /* for init_sentry */
 #include "pluginlaunch.h"          /* for init_loading_shm */
 #include "processes.h"             /* for create_process */
 #include "sighand.h"               /* for openvas_signal */
 #include "utils.h"                 /* for store_file */
 
-#include <errno.h>  /* for errno() */
-#include <fcntl.h>  /* for open() */
-#include <gcrypt.h> /* for gcry_control */
+#include <bsd/unistd.h> /* for proctitle_init */
+#include <errno.h>      /* for errno() */
+#include <fcntl.h>      /* for open() */
+#include <gcrypt.h>     /* for gcry_control */
 #include <glib.h>
 #include <gnutls/gnutls.h> /* for gnutls_global_set_log_*  */
 #include <grp.h>
 #include <gvm/base/logging.h> /* for setup_log_handler, load_log_configuration, free_log_configuration*/
 #include <gvm/base/nvti.h>      /* for prefs_get() */
 #include <gvm/base/prefs.h>     /* for prefs_get() */
-#include <gvm/base/proctitle.h> /* for proctitle_set */
 #include <gvm/base/version.h>   /* for gvm_libs_version */
 #include <gvm/util/kb.h>        /* for KB_PATH_DEFAULT */
+#include <gvm/util/mqtt.h>      /* for mqtt_init */
 #include <gvm/util/nvticache.h> /* nvticache_free */
 #include <gvm/util/uuidutils.h> /* gvm_uuid_make */
 #include <netdb.h>              /* for addrinfo */
@@ -99,7 +105,7 @@ int global_max_sysload = 0;
 GSList *log_config = NULL;
 
 static volatile int termination_signal = 0;
-static char *global_scan_id = NULL;
+// static char *global_scan_id = NULL;
 
 typedef struct
 {
@@ -121,6 +127,16 @@ static openvas_option openvas_defaults[] = {
   {"db_address", KB_PATH_DEFAULT},
   {NULL, NULL}};
 
+/**
+ * @brief Set the prefs from the openvas_defaults array.
+ */
+static void
+set_default_openvas_prefs ()
+{
+  for (int i = 0; openvas_defaults[i].option != NULL; i++)
+    prefs_set (openvas_defaults[i].option, openvas_defaults[i].value);
+}
+
 static void
 my_gnutls_log_func (int level, const char *text)
 {
@@ -161,20 +177,11 @@ set_globals_from_preferences (void)
     }
 }
 
-static void
-reload_openvas (void);
-
-static void
-handle_reload_signal (int sig)
-{
-  (void) sig;
-  reload_openvas ();
-}
-
 static void
 handle_termination_signal (int sig)
 {
   termination_signal = sig;
+  procs_terminate_childs ();
 }
 
 /**
@@ -186,56 +193,24 @@ init_signal_handlers (void)
   openvas_signal (SIGTERM, handle_termination_signal);
   openvas_signal (SIGINT, handle_termination_signal);
   openvas_signal (SIGQUIT, handle_termination_signal);
-  openvas_signal (SIGHUP, handle_reload_signal);
   openvas_signal (SIGCHLD, sighand_chld);
 }
 
-/* Restarts the scanner by reloading the configuration. */
-static void
-reload_openvas (void)
-{
-  static gchar *rc_name = NULL;
-  const char *config_file;
-  int i, ret;
-
-  /* Ignore SIGHUP while reloading. */
-  openvas_signal (SIGHUP, SIG_IGN);
-
-  proctitle_set (PROCTITLE_RELOADING);
-  /* Setup logging. */
-  rc_name = g_build_filename (OPENVAS_SYSCONF_DIR, "openvas_log.conf", NULL);
-  if (g_file_test (rc_name, G_FILE_TEST_EXISTS))
-    log_config = load_log_configuration (rc_name);
-  g_free (rc_name);
-  setup_log_handlers (log_config);
-  g_message ("Reloading the scanner.\n");
-
-  /* Reload config file. */
-  config_file = prefs_get ("config_file");
-  for (i = 0; openvas_defaults[i].option != NULL; i++)
-    prefs_set (openvas_defaults[i].option, openvas_defaults[i].value);
-  prefs_config (config_file);
-
-  /* Reload the plugins */
-  ret = plugins_init ();
-  set_globals_from_preferences ();
-
-  g_message ("Finished reloading the scanner.");
-  openvas_signal (SIGHUP, handle_reload_signal);
-  proctitle_set (PROCTITLE_WAITING);
-  if (ret)
-    exit (1);
-}
-
 /**
  * @brief Read the scan preferences from redis
- * @input scan_id Scan ID used as key to find the corresponding KB where
- *                to take the preferences from.
+ *
+ * Adds preferences to the global_prefs.
+ * If preference already exists in global_prefs they will be overwritten by
+ * prefs from client.
+ *
+ * @param globals Scan ID of globals used as key to find the corresponding KB
+ * where to take the preferences from. Globals also used for file upload.
+ *
  * @return 0 on success, -1 if the kb is not found or no prefs are found in
  *         the kb.
  */
 static int
-load_scan_preferences (struct scan_globals *globals)
+overwrite_openvas_prefs_with_prefs_from_client (struct scan_globals *globals)
 {
   char key[1024];
   kb_t kb;
@@ -246,9 +221,13 @@ load_scan_preferences (struct scan_globals *globals)
     return -1;
 
   snprintf (key, sizeof (key), "internal/%s/scanprefs", globals->scan_id);
+
   kb = kb_find (prefs_get ("db_address"), key);
   if (!kb)
     return -1;
+  // 2022-10-19: currently internal/%s/scanprefs are set by ospd which is the
+  // main_kb in our context
+  set_main_kb (kb);
 
   res = kb_item_get_all (kb, key);
   if (!res)
@@ -263,23 +242,23 @@ load_scan_preferences (struct scan_globals *globals)
           if (pref_name[1] && pref_name[2] && !strncmp (pref_name[2], "file", 4)
               && strcmp (pref[1], ""))
             {
-              char *file_hash = gvm_uuid_make ();
+              char *file_uuid = gvm_uuid_make ();
               int ret;
-              prefs_set (pref[0], file_hash);
-              ret = store_file (globals, pref[1], file_hash);
+              prefs_set (pref[0], file_uuid);
+              ret = store_file (globals, pref[1], file_uuid);
               if (ret)
                 g_debug ("Load preference: Failed to upload file "
                          "for nvt %s preference.",
                          pref_name[0]);
 
-              g_free (file_hash);
+              g_free (file_uuid);
             }
           else if (is_scanner_only_pref (pref[0]))
             g_warning ("%s is a scanner only preference. It can not be written "
                        "by the client and will be ignored.",
                        pref_name[0]);
           else
-            prefs_set (pref[0], pref[1] ?: "");
+            prefs_set (pref[0], pref[1] ? pref[1] : "");
           g_strfreev (pref_name);
         }
 
@@ -288,8 +267,8 @@ load_scan_preferences (struct scan_globals *globals)
     }
   kb_del_items (kb, key);
   snprintf (key, sizeof (key), "internal/%s", globals->scan_id);
-  kb_item_set_str (kb, key, "ready", 0);
-  kb_item_set_int (kb, "internal/ovas_pid", getpid ());
+  kb_item_set_str_with_main_kb_check (kb, key, "ready", 0);
+  kb_item_set_int_with_main_kb_check (kb, "internal/ovas_pid", getpid ());
   kb_lnk_reset (kb);
 
   g_debug ("End loading scan preferences.");
@@ -298,59 +277,31 @@ load_scan_preferences (struct scan_globals *globals)
   return 0;
 }
 
-static void
-scanner_thread (struct scan_globals *globals)
-{
-  /* Make process a group leader, to make it easier to cleanup forked
-   * processes & their children. */
-  setpgid (0, 0);
-  nvticache_reset ();
-
-  globals->scan_id = g_strdup (global_scan_id);
-
-  /* Load preferences from Redis. Scan started with a scan_id. */
-  if (load_scan_preferences (globals))
-    {
-      g_warning ("No preferences found for the scan %s", globals->scan_id);
-      exit (0);
-    }
-
-  attack_network (globals);
-
-  exit (0);
-}
-
 /**
- * @brief Initialize everything.
+ * @brief Init logging.
  *
- * @param config_file Path to config file for initialization
+ * @return 0 on success, -1 on error.
  */
 static int
-init_openvas (const char *config_file)
+init_logging ()
 {
-  static gchar *rc_name = NULL;
-  int i;
+  static gchar *log_config_file_name = NULL;
   int err;
 
-  for (i = 0; openvas_defaults[i].option != NULL; i++)
-    prefs_set (openvas_defaults[i].option, openvas_defaults[i].value);
-  prefs_config (config_file);
-
-  /* Setup logging. */
-  rc_name = g_build_filename (OPENVAS_SYSCONF_DIR, "openvas_log.conf", NULL);
-  if (g_file_test (rc_name, G_FILE_TEST_EXISTS))
-    log_config = load_log_configuration (rc_name);
+  log_config_file_name =
+    g_build_filename (OPENVAS_SYSCONF_DIR, "openvas_log.conf", NULL);
+  if (g_file_test (log_config_file_name, G_FILE_TEST_EXISTS))
+    log_config = load_log_configuration (log_config_file_name);
   err = setup_log_handlers (log_config);
   if (err)
     {
       g_warning ("%s: Can not open or create log file or directory. "
                  "Please check permissions of log files listed in %s.",
-                 __func__, rc_name);
-      g_free (rc_name);
+                 __func__, log_config_file_name);
+      g_free (log_config_file_name);
       return -1;
     }
-  g_free (rc_name);
-  set_globals_from_preferences ();
+  g_free (log_config_file_name);
 
   return 0;
 }
@@ -367,25 +318,12 @@ gcrypt_init (void)
   gcry_control (GCRYCTL_INITIALIZATION_FINISHED);
 }
 
-void
-send_message_to_client_and_finish_scan (const char *msg)
+/**
+ * @brief Check TLS.
+ */
+static void
+check_tls ()
 {
-  char key[1024];
-  kb_t kb;
-
-  snprintf (key, sizeof (key), "internal/%s/scanprefs", global_scan_id);
-  kb = kb_find (prefs_get ("db_address"), key);
-  kb_item_push_str (kb, "internal/results", msg);
-  snprintf (key, sizeof (key), "internal/%s", global_scan_id);
-  kb_item_set_str (kb, key, "finished", 0);
-  kb_lnk_reset (kb);
-}
-
-void
-start_single_task_scan (void)
-{
-  struct scan_globals *globals;
-
 #if GNUTLS_VERSION_NUMBER < 0x030300
   if (openvas_SSL_init () < 0)
     g_message ("Could not initialize openvas SSL!");
@@ -399,62 +337,142 @@ start_single_task_scan (void)
       gnutls_global_set_log_function (my_gnutls_log_func);
       gnutls_global_set_log_level (atoi (prefs_get ("debug_tls")));
     }
+}
 
+/**
+ * @brief Print start message.
+ */
+static void
+openvas_print_start_msg ()
+{
 #ifdef OPENVAS_GIT_REVISION
   g_message ("openvas %s (GIT revision %s) started", OPENVAS_VERSION,
              OPENVAS_GIT_REVISION);
 #else
   g_message ("openvas %s started", OPENVAS_VERSION);
 #endif
-
-  if (plugins_cache_init ())
-    {
-      g_message ("Failed to initialize nvti cache.");
-      send_message_to_client_and_finish_scan (
-        "ERRMSG||| ||| ||| ||| |||NVTI cache initialization failed");
-      exit (1);
-    }
-  init_signal_handlers ();
-
-  globals = g_malloc0 (sizeof (struct scan_globals));
-
-  scanner_thread (globals);
-  exit (0);
 }
+
 /**
  * @brief Search in redis the process ID of a running scan and
  * sends it the kill signal SIGUSR1, which will stop the scan.
  * To find the process ID, it uses the scan_id passed with the
  * --scan-stop option.
+ *
+ * @return 0 on success, 1 otherwise.
  */
-static void
+static int
 stop_single_task_scan (void)
 {
   char key[1024];
   kb_t kb;
   int pid;
 
-  if (!global_scan_id)
-    exit (1);
+  if (!get_scan_id ())
+    return 1;
 
-  snprintf (key, sizeof (key), "internal/%s", global_scan_id);
+  snprintf (key, sizeof (key), "internal/%s", get_scan_id ());
   kb = kb_find (prefs_get ("db_address"), key);
   if (!kb)
-    exit (1);
+    return 1;
 
   pid = kb_item_get_int (kb, "internal/ovas_pid");
 
   /* Only send the signal if the pid is a positive value.
      Since kb_item_get_int() will return -1 if the key does
-     not exist. killing with -1 pid will send the signal system wide.
+     not exist.
+     Warning: killing with -1 pid will send the signal system wide.
    */
   if (pid <= 0)
-    return;
+    return 1;
 
   /* Send the signal to the process group. */
   killpg (pid, SIGUSR1);
+  return 0;
+}
 
-  exit (0);
+/**
+ * @brief Send a failure message and set the scan as finished.
+ *
+ * @param msg Message to send to the client.
+ */
+static void
+send_message_to_client_and_finish_scan (const char *msg)
+{
+  char key[1024];
+  kb_t kb;
+
+  // We get the main kb. It is still not set as global at this point.
+  snprintf (key, sizeof (key), "internal/%s/scanprefs", get_scan_id ());
+  kb = kb_find (prefs_get ("db_address"), key);
+  kb_item_push_str (kb, "internal/results", msg);
+  snprintf (key, sizeof (key), "internal/%s", get_scan_id ());
+  kb_item_set_str (kb, key, "finished", 0);
+  kb_lnk_reset (kb);
+}
+
+/**
+ * @brief Set up data needed for attack_network().
+ *
+ * @param globals scan_globals needed for client preference handling.
+ * @param config_file Used for config preference handling.
+ *
+ * @return 0 on success, 1 otherwise.
+ */
+static int
+attack_network_init (struct scan_globals *globals, const gchar *config_file)
+{
+  const char *mqtt_server_uri;
+
+  set_default_openvas_prefs ();
+  prefs_config (config_file);
+  set_globals_from_preferences ();
+
+  if (prefs_get ("vendor_version") != NULL)
+    vendor_version_set (prefs_get ("vendor_version"));
+  check_tls ();
+  openvas_print_start_msg ();
+
+  if (plugins_cache_init ())
+    {
+      g_message ("Failed to initialize nvti cache.");
+      send_message_to_client_and_finish_scan (
+        "ERRMSG||| ||| ||| ||| |||NVTI cache initialization failed");
+      nvticache_reset ();
+      return 1;
+    }
+  nvticache_reset ();
+
+  /* Init MQTT communication */
+  mqtt_server_uri = prefs_get ("mqtt_server_uri");
+  if (mqtt_server_uri)
+    {
+      if ((mqtt_init (mqtt_server_uri)) != 0)
+        {
+          g_message ("%s: INIT MQTT: FAIL", __func__);
+          send_message_to_client_and_finish_scan (
+            "ERRMSG||| ||| ||| ||| |||MQTT initialization failed");
+        }
+      else
+        {
+          g_message ("%s: INIT MQTT: SUCCESS", __func__);
+          prefs_set ("mqtt_enabled", "yes");
+        }
+    }
+
+  init_signal_handlers ();
+
+  /* Make process a group leader, to make it easier to cleanup forked
+   * processes & their children. */
+  setpgid (0, 0);
+
+  if (overwrite_openvas_prefs_with_prefs_from_client (globals))
+    {
+      g_warning ("No preferences found for the scan %s", globals->scan_id);
+      return 1;
+    }
+
+  return 0;
 }
 
 /**
@@ -463,11 +481,11 @@ stop_single_task_scan (void)
  * @param argv Argument vector.
  */
 int
-openvas (int argc, char *argv[])
+openvas (int argc, char *argv[], char *env[])
 {
-  int ret;
+  int err;
 
-  proctitle_init (argc, argv);
+  setproctitle_init (argc, argv, env);
   gcrypt_init ();
 
   static gboolean display_version = FALSE;
@@ -505,7 +523,7 @@ openvas (int argc, char *argv[])
   if (!g_option_context_parse (option_context, &argc, &argv, &error))
     {
       g_print ("%s\n\n", error->message);
-      exit (0);
+      return EXIT_SUCCESS;
     }
   g_option_context_free (option_context);
 
@@ -513,7 +531,7 @@ openvas (int argc, char *argv[])
   if (print_sysconfdir)
     {
       g_print ("%s\n", SYSCONFDIR);
-      exit (0);
+      return EXIT_SUCCESS;
     }
 
   /* --version */
@@ -524,73 +542,95 @@ openvas (int argc, char *argv[])
       printf ("GIT revision %s\n", OPENVAS_GIT_REVISION);
 #endif
       printf ("gvm-libs %s\n", gvm_libs_version ());
-      printf ("Most new code since 2005: (C) 2021 Greenbone Networks GmbH\n");
+      printf ("Most new code since 2005: (C) 2022 Greenbone Networks GmbH\n");
       printf (
         "Nessus origin: (C) 2004 Renaud Deraison <deraison@nessus.org>\n");
       printf ("License GPLv2: GNU GPL version 2\n");
       printf (
         "This is free software: you are free to change and redistribute it.\n"
         "There is NO WARRANTY, to the extent permitted by law.\n\n");
-      exit (0);
+      return EXIT_SUCCESS;
     }
 
   /* Switch to UTC so that OTP times are always in UTC. */
   if (setenv ("TZ", "utc 0", 1) == -1)
     {
       g_print ("%s\n\n", strerror (errno));
-      exit (0);
+      return EXIT_SUCCESS;
     }
   tzset ();
 
+  if (init_logging () != 0)
+    return EXIT_FAILURE;
+
+  if (!init_sentry ())
+    {
+      g_message ("Sentry is enabled. This can log sensitive information.");
+    }
+
+  /* Config file location */
   if (!config_file)
     config_file = OPENVAS_CONF;
+
   if (update_vt_info)
     {
-      if (init_openvas (config_file))
-        return 1;
-      if (plugins_init ())
-        return 1;
-      return 0;
+      set_default_openvas_prefs ();
+      prefs_config (config_file);
+      set_globals_from_preferences ();
+      err = plugins_init ();
+      nvticache_reset ();
+      gvm_close_sentry ();
+      return err ? EXIT_FAILURE : EXIT_SUCCESS;
     }
 
-  if (init_openvas (config_file))
-    return 1;
-
-  if (prefs_get ("vendor_version") != NULL)
-    vendor_version_set (prefs_get ("vendor_version"));
-
+  /* openvas --scan-stop */
   if (stop_scan_id)
     {
-      global_scan_id = g_strdup (stop_scan_id);
-      stop_single_task_scan ();
-      exit (0);
+      set_default_openvas_prefs ();
+      prefs_config (config_file);
+      if (plugins_cache_init ())
+        {
+          g_message ("Failed to initialize nvti cache. Not possible to "
+                     "stop the scan");
+          nvticache_reset ();
+          gvm_close_sentry ();
+          return EXIT_FAILURE;
+        }
+      nvticache_reset ();
+
+      set_scan_id (g_strdup (stop_scan_id));
+      err = stop_single_task_scan ();
+      gvm_close_sentry ();
+      return err ? EXIT_FAILURE : EXIT_SUCCESS;
     }
 
+  /* openvas --scan-start */
   if (scan_id)
     {
-      global_scan_id = g_strdup (scan_id);
-      start_single_task_scan ();
-      exit (0);
+      struct scan_globals *globals;
+      set_scan_id (g_strdup (scan_id));
+      globals = g_malloc0 (sizeof (struct scan_globals));
+      globals->scan_id = g_strdup (get_scan_id ());
+
+      if (attack_network_init (globals, config_file) != 0)
+        {
+          destroy_scan_globals (globals);
+          return EXIT_FAILURE;
+        }
+      attack_network (globals);
+
+      gvm_close_sentry ();
+      destroy_scan_globals (globals);
+      return EXIT_SUCCESS;
     }
 
-  /* special treatment */
   if (print_specs)
     {
+      set_default_openvas_prefs ();
+      prefs_config (config_file);
       prefs_dump ();
-      exit (0);
+      gvm_close_sentry ();
     }
 
-#if GNUTLS_VERSION_NUMBER < 0x030300
-  if (openvas_SSL_init () < 0)
-    g_message ("Could not initialize openvas SSL!");
-#endif
-
-  /* Ignore SIGHUP while reloading. */
-  openvas_signal (SIGHUP, SIG_IGN);
-
-  ret = plugins_init ();
-  if (ret)
-    return 1;
-
-  exit (0);
+  return EXIT_SUCCESS;
 }
