@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2021 Greenbone Networks GmbH
+/* Copyright (C) 2019-2022 Greenbone Networks GmbH
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
@@ -22,10 +22,10 @@
  * @brief Headers for OpenVAS entry point.
  */
 
-#ifndef _OPENVAS_H
-#define _OPENVAS_H
+#ifndef OPENVAS_H
+#define OPENVAS_H
 
 int
-openvas (int, char **);
+openvas (int, char **, char **);
 
-#endif /* not _OPENVAS_H */
+#endif /* not OPENVAS_H */
