@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -27,6 +27,7 @@
 
 #include "../misc/network.h"
 #include "../misc/nvt_categories.h" /* for ACT_SCANNER */
+#include "../misc/plugutils.h"      /* for get_plugin_preference */
 #include "pluginload.h"
 #include "pluginscheduler.h"
 #include "plugs_req.h"
@@ -78,11 +79,46 @@ static int old_max_running_processes;
 static GSList *non_simult_ports = NULL;
 const char *hostname = NULL;
 
+/**
+ * @brief Check if max_nvt_timeouts is set and if has been reached
+ *
+ * @return 1 if reached, 0 if not reached or no set.
+ */
+static int
+max_nvt_timeouts_reached ()
+{
+  static int vts_timeouts_counter = 0;
+  int max_vts_timeouts = 0;
+  const gchar *max_vts_timeouts_str = NULL;
+
+  /* Check if set */
+  if ((max_vts_timeouts_str = prefs_get ("max_vts_timeouts")) == NULL)
+    {
+      g_debug ("%s: max_vts_timeouts not set.", __func__);
+      return 0;
+    }
+
+  /* Check if enabled and valid value */
+  max_vts_timeouts = atoi (max_vts_timeouts_str);
+  if (max_vts_timeouts <= 0)
+    {
+      g_debug ("%s: max_vts_timeouts disabled", __func__);
+      return 0;
+    }
+
+  vts_timeouts_counter++;
+  /* Check if reached */
+  if (vts_timeouts_counter >= max_vts_timeouts)
+    return 1;
+
+  return 0;
+}
+
 /**
  *
  */
 static void
-update_running_processes (kb_t kb)
+update_running_processes (kb_t main_kb, kb_t kb)
 {
   int i;
   struct timeval now;
@@ -107,24 +143,43 @@ update_running_processes (kb_t kb)
             {
               char *oid = processes[i].plugin->oid;
 
-              if (is_alive)
+              if (is_alive) // Alive and timed out
                 {
                   char msg[2048];
-
                   if (log_whole)
                     g_message ("%s (pid %d) is slow to finish - killing it",
                                oid, processes[i].pid);
 
-                  sprintf (msg,
-                           "ERRMSG|||%s||| |||general/tcp|||%s|||"
-                           "NVT timed out after %d seconds.",
-                           hostname, oid ?: " ", processes[i].timeout);
+                  g_snprintf (msg, sizeof (msg),
+                              "ERRMSG|||%s||| |||general/tcp|||%s|||"
+                              "NVT timed out after %d seconds.",
+                              hostname, oid ? oid : " ", processes[i].timeout);
+                  kb_item_push_str_with_main_kb_check (main_kb,
+                                                       "internal/results", msg);
 
-                  kb_item_push_str (kb, "internal/results", msg);
+                  /* Check for max VTs timeouts */
+                  if (max_nvt_timeouts_reached ())
+                    {
+                      /* Check if host is still alive and send a message
+                         if it is dead. */
+                      if (check_host_still_alive (kb, hostname) == 0)
+                        {
+                          g_snprintf (msg, sizeof (msg),
+                                      "ERRMSG|||%s||| |||general/tcp||| |||"
+                                      "Host has been marked as dead. Too many "
+                                      "NVT_TIMEOUTs.",
+                                      hostname);
+                          kb_item_push_str_with_main_kb_check (
+                            main_kb, "internal/results", msg);
+                        }
+                    }
 
                   ret_terminate = terminate_process (processes[i].pid);
                   if (ret_terminate == 0)
                     {
+                      /* Since the plugin process is a group leader process
+                       * we can send the signal to -PID process to kill
+                       * also the plugin's child processes. */
                       terminate_process (processes[i].pid * -1);
                       num_running_processes--;
                       processes[i].plugin->running_state = PLUGIN_STATUS_DONE;
@@ -158,6 +213,9 @@ update_running_processes (kb_t kb)
                     }
                   while (e < 0 && errno == EINTR);
 
+                  /* Since the plugin process is a group leader process
+                   * we can send the signal to -PID process to kill
+                   * also the plugin's child processes. */
                   terminate_process (processes[i].pid * -1);
                   num_running_processes--;
                   processes[i].plugin->running_state = PLUGIN_STATUS_DONE;
@@ -243,7 +301,7 @@ simult_ports (const char *oid, const char *next_oid)
  * free "slot" in the processes array otherwise.
  */
 static int
-next_free_process (kb_t kb, struct scheduler_plugin *upcoming)
+next_free_process (kb_t main_kb, kb_t kb, struct scheduler_plugin *upcoming)
 {
   int r;
 
@@ -254,7 +312,7 @@ next_free_process (kb_t kb, struct scheduler_plugin *upcoming)
         {
           while (process_alive (processes[r].pid))
             {
-              update_running_processes (kb);
+              update_running_processes (main_kb, kb);
               usleep (250000);
             }
         }
@@ -311,6 +369,9 @@ pluginlaunch_stop (void)
     {
       if (processes[i].pid > 0)
         {
+          /* Since the plugin process is a group leader process
+           * we can send the signal to -PID process to kill
+           * also the plugin's child processes. */
           terminate_process (processes[i].pid * -1);
           num_running_processes--;
           processes[i].plugin->running_state = PLUGIN_STATUS_DONE;
@@ -322,19 +383,26 @@ pluginlaunch_stop (void)
 static int
 plugin_timeout (nvti_t *nvti)
 {
-  int timeout;
+  int timeout, tmp;
+  gchar *timeout_str;
+
+  timeout = 0;
+  if ((timeout_str = get_plugin_preference (nvti_oid (nvti), "timeout", 0))
+      != NULL)
+    timeout = atoi (timeout_str);
 
-  assert (nvti);
-  timeout = prefs_nvt_timeout (nvti_oid (nvti));
-  if (timeout == 0)
-    timeout = nvti_timeout (nvti);
   if (timeout == 0)
     {
       if (nvti_category (nvti) == ACT_SCANNER)
-        timeout =
-          atoi (prefs_get ("scanner_plugins_timeout")) ?: SCANNER_NVT_TIMEOUT;
+        {
+          tmp = atoi (prefs_get ("scanner_plugins_timeout"));
+          timeout = tmp ? tmp : SCANNER_NVT_TIMEOUT;
+        }
       else
-        timeout = atoi (prefs_get ("plugins_timeout")) ?: NVT_TIMEOUT;
+        {
+          tmp = atoi (prefs_get ("plugins_timeout"));
+          timeout = tmp ? tmp : NVT_TIMEOUT;
+        }
     }
   return timeout;
 }
@@ -375,7 +443,7 @@ check_memory ()
   return 1;
 }
 
-int
+static int
 check_sysload ()
 {
   double sysload;
@@ -406,8 +474,8 @@ plugin_launch (struct scan_globals *globals, struct scheduler_plugin *plugin,
   int p;
 
   /* Wait for a free slot */
-  pluginlaunch_wait_for_free_process (main_kb);
-  p = next_free_process (main_kb, plugin);
+  pluginlaunch_wait_for_free_process (main_kb, kb);
+  p = next_free_process (main_kb, kb, plugin);
   if (p < 0)
     {
       g_warning ("%s. There is currently no free slot available for starting a "
@@ -420,8 +488,7 @@ plugin_launch (struct scan_globals *globals, struct scheduler_plugin *plugin,
   processes[p].plugin = plugin;
   processes[p].timeout = plugin_timeout (nvti);
   gettimeofday (&(processes[p].start), NULL);
-  processes[p].pid =
-    nasl_plugin_launch (globals, ip, vhosts, kb, main_kb, plugin->oid);
+  processes[p].pid = nasl_plugin_launch (globals, ip, vhosts, kb, plugin->oid);
 
   if (processes[p].pid > 0)
     num_running_processes++;
@@ -437,11 +504,11 @@ plugin_launch (struct scan_globals *globals, struct scheduler_plugin *plugin,
  * @brief Waits and 'pushes' processes until num_running_processes is 0.
  */
 void
-pluginlaunch_wait (kb_t kb)
+pluginlaunch_wait (kb_t main_kb, kb_t kb)
 {
   while (num_running_processes)
     {
-      update_running_processes (kb);
+      update_running_processes (main_kb, kb);
       if (num_running_processes)
         waitpid (-1, NULL, 0);
     }
@@ -470,11 +537,11 @@ timeout_running_processes (void)
  *        changed.
  */
 void
-pluginlaunch_wait_for_free_process (kb_t kb)
+pluginlaunch_wait_for_free_process (kb_t main_kb, kb_t kb)
 {
   if (!num_running_processes)
     return;
-  update_running_processes (kb);
+  update_running_processes (main_kb, kb);
   /* Max number of processes are still running, wait for a child to exit or
    * to timeout. */
 
@@ -484,6 +551,9 @@ pluginlaunch_wait_for_free_process (kb_t kb)
              "Waiting for free slot for processes.",
              __func__, num_running_processes, max_running_processes);
 
+  /* Be careful with changing the max_running_process value.
+   * The plugin scheduler can change this value for running one plugin at
+   * time. */
   while (
     (num_running_processes >= max_running_processes)
     || (num_running_processes > 0 && (check_memory () || check_sysload ())))
@@ -495,8 +565,24 @@ pluginlaunch_wait_for_free_process (kb_t kb)
       assert (ts.tv_sec);
       sigemptyset (&mask);
       sigaddset (&mask, SIGCHLD);
-      if (sigtimedwait (&mask, NULL, &ts) < 0 && errno != EAGAIN)
-        g_warning ("%s: %s", __func__, strerror (errno));
-      update_running_processes (kb);
+      sigaddset (&mask, SIGUSR1);
+      /* Wait here for the shortest plugins timeout or for a child which ended.
+       * Also, it handles signal SIGUSR1 to stop a scan. Otherwise the signa is
+       * ignored, the plugin is never stopped and the scanner keeps waiting. */
+      int sig = sigtimedwait (&mask, NULL, &ts);
+      if (sig < 0 && errno != EAGAIN)
+        g_warning ("%s: %s (%d)", __func__, strerror (errno), errno);
+      else if (sig == SIGUSR1)
+        {
+          /* SIGUSR1 signal is sent during scan stop to all host processes.
+             Therefore pluginlaunch_stop() is called here, for the
+             special case in which we are waiting for the last plugin, of the
+             last host, to finish.
+          */
+          pluginlaunch_stop ();
+        }
+      // cleanup ipcc cache
+      procs_cleanup_children ();
+      update_running_processes (main_kb, kb);
     }
 }
