@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -23,8 +23,8 @@
  * @brief plugs_req.c header.
  */
 
-#ifndef PLUGINS_REQUIREMENTS_H__
-#define PLUGINS_REQUIREMENTS_H__
+#ifndef OPENVAS_PLUGS_REQ_H
+#define OPENVAS_PLUGS_REQ_H
 
 #include <gvm/util/kb.h> /* for struct kb_item */
 
