@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -23,14 +23,32 @@
  * @brief processes.c header.
  */
 
-#ifndef _OPENVAS_THREADS_H
-#define _OPENVAS_THREADS_H
+#ifndef OPENVAS_PROCESSES_H
+#define OPENVAS_PROCESSES_H
+
+#include "../misc/ipc.h"
 
 #include <sys/types.h> /* for pid_t */
 
+#define FORKFAILED -1
+#define NOINIT -2
+#define PROCSFULL -3
+#define NOCHILD -4
+
 typedef void (*process_func_t) (void *);
+
+void
+procs_terminate_childs (void);
+
+int
+terminate_process (pid_t pid);
+
 pid_t
-create_process (process_func_t, void *);
-int terminate_process (pid_t);
+create_ipc_process (ipc_process_func func, void *args);
+const struct ipc_contexts *
+procs_get_ipc_contexts (void);
+
+int
+procs_cleanup_children (void);
 
 #endif
