@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -23,8 +23,8 @@
  * @brief headerfile for sighand.c.
  */
 
-#ifndef _OPENVAS_SIGHAND_H
-#define _OPENVAS_SIGHAND_H
+#ifndef OPENVAS_SIGHAND_H
+#define OPENVAS_SIGHAND_H
 
 void (*openvas_signal (int signum, void (*handler) (int))) (int);
 void
