@@ -1,4 +1,4 @@
-/* Portions Copyright (C) 2009-2021 Greenbone Networks GmbH
+/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
  * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
  * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
  *
@@ -23,8 +23,10 @@
  * @brief utils.c headerfile.
  */
 
-#ifndef _OPENVAS_UTILS_H
-#define _OPENVAS_UTILS_H
+#ifndef OPENVAS_UTILS_H
+#define OPENVAS_UTILS_H
+
+#include "../misc/scanneraux.h"
 
 #include <sys/types.h> /* for pid_t */
 
@@ -46,6 +48,9 @@ int
 is_scanner_only_pref (const char *);
 
 int
-store_file (struct scan_globals *, const char *, const char *);
+store_file (struct scan_globals *globals, const char *file,
+            const char *file_hash);
 
+int
+check_host_still_alive (kb_t, const char *);
 #endif
